@@ -45,6 +45,7 @@ rsync.checkgauge = {
 	filter      =  true,
 	filterFrom  =  true,
 	target      =  true,
+	batchSizeLimit = true,
 
 	rsync  = {
 		acls              =  true,
@@ -61,7 +62,9 @@ rsync.checkgauge = {
 		compress          =  true,
 		copy_dirlinks     =  true,
 		copy_links        =  true,
+		copy_unsafe_links =  true,
 		cvs_exclude       =  true,
+		delete_excluded   =  true,
 		dry_run           =  true,
 		executability     =  true,
 		existing          =  true,
@@ -100,36 +103,16 @@ rsync.checkgauge = {
 }
 
 
---
--- Returns true for non Init and Blanket events.
---
-local eventNotInitBlank =
-	function
-(
-	event
-)
-	return event.etype ~= 'Init' and event.etype ~= 'Blanket'
-end
-
-
---
--- Spawns rsync for a list of events
---
--- Exclusions are already handled by not having
--- events for them.
---
-rsync.action = function
-(
-	inlet
-)
+-- internal function to actually do the transfer
+local run_action = function
+	(
+		inlet,
+		elist
+	)
 	local config = inlet.getConfig( )
 
-	-- gets all events ready for syncing
-	local elist = inlet.getEvents( eventNotInitBlank )
-
-	-- gets the list of paths for the event list
-	-- deletes create multi match patterns
-	local paths = elist.getPaths( )
+	local substitudes = inlet.getSubstitutionData(elist, {})
+	local target = substitudeCommands(config.target, substitudes)
 
 	--
 	-- Replaces what rsync would consider filter rules by literals
@@ -235,11 +218,126 @@ rsync.action = function
 		'--include-from=-',
 		'--exclude=*',
 		config.source,
-		config.target
+		target
 	)
 end
 
 
+--
+-- Returns true for non Init and Blanket events.
+--
+local eventNotInitBlank =
+       function
+(
+       event
+)
+       return event.etype ~= 'Init' and event.etype ~= 'Blanket'
+end
+
+--
+-- Returns size or true if the event is for batch processing
+--
+local getBatchSize =
+	function
+(
+	event
+)
+	-- print("getBatchSize", event, event.status, event.etype, event.pathname)
+	if event.status == 'active' then
+		return false
+	end
+	if event.etype == 'Init' or event.etype == 'Blanket' or event.etype == 'Full' then
+		return false
+	end
+	-- moves and deletes go always into batch
+	if event.etype == 'Move' or event.etype == 'Delete' then
+		return true
+	end
+	return lsyncd.get_file_size(event.sourcePath)
+end
+
+--
+-- Spawns rsync for a list of events
+--
+-- Exclusions are already handled by not having
+-- events for them.
+--
+rsync.action = function
+	(
+		inlet
+	)
+	local sizeLimit = inlet.getConfig().batchSizeLimit
+
+	if sizeLimit == nil then
+		-- gets all events ready for syncing
+		return run_action(inlet, inlet.getEvents(eventNotInitBlank))
+	else
+		-- spawn all files under the size limit/deletes/moves in batch mode
+		local eventInBatch = function(event)
+			if event.etype == "Full" then
+				return false
+			end
+			local size = getBatchSize(event)
+			if type(size) == "boolean" then
+				return size
+			elseif size == nil then
+				return true
+			end
+			if size <= sizeLimit then
+				return true
+			end
+			return false
+		end
+
+		-- indicator for grabbing one element of the queue
+		local single_returned = false
+		-- grab all events for seperate transfers
+		local eventNoBatch = function(event)
+			if event.etype == "Full" then
+				return false
+			end
+			local size = getBatchSize(event)
+			if type(size) ~= "number" or size == nil then
+				return false
+			end
+			if single_returned then
+				return 'break'
+			end
+			if size > sizeLimit then
+				single_returned = true
+				return true
+			end
+			return false
+		end
+		local extralist = inlet.getEvents(eventInBatch)
+
+		-- get all batched events
+		if extralist.size() > 0 then
+			run_action(inlet, extralist)
+		end
+
+		while true do
+			local cnt, maxcnt = lsyncd.get_process_info()
+			if inlet.getSync().processes:size( ) >= inlet.getConfig().maxProcesses then
+				log('Normal',
+				'Maximum processes for sync reached. Delaying large transfer for sync: '..inlet.getConfig().name)
+				break
+			elseif maxcnt and cnt >= maxcnt then
+			log('Normal',
+				'Maximum process count reached. Delaying large transfer for sync: '..inlet.getConfig().name)
+				break
+			end
+			local extralist = inlet.getEvents(eventNoBatch)
+
+			-- no more single size events
+			if extralist.size() == 0 then break end
+			run_action(inlet, extralist)
+			-- get next result
+			single_returned = false
+		end
+	end
+end
+
 ----
 ---- NOTE: This optimized version can be used once
 ----       https://bugzilla.samba.org/show_bug.cgi?id=12569
@@ -308,6 +406,16 @@ rsync.init = function
 (
 	event
 )
+	return rsync.full(event)
+end
+
+--
+-- Triggers a full sync event
+--
+rsync.full = function
+	(
+		event
+	)
 	local config   = event.config
 
 	local inlet    = event.inlet
@@ -316,7 +424,7 @@ rsync.init = function
 
 	local filters = inlet.hasFilters( ) and inlet.getFilters( )
 
-	local delete   = nil
+	local delete   = {}
 
 	local target   = config.target
 
@@ -330,18 +438,26 @@ rsync.init = function
 		target = config.host .. ':' .. config.targetdir
 	end
 
+	local substitudes = inlet.getSubstitutionData(event, {})
+	target = substitudeCommands(target, substitudes)
+
 	if config.delete == true
 	or config.delete == 'startup'
 	then
 		delete = { '--delete', '--ignore-errors' }
 	end
 
+	if config.rsync.delete_excluded == true
+	then
+		table.insert( delete, '--delete-excluded' )
+	end
+
 	if not filters and #excludes == 0
 	then
 		-- starts rsync without any filters or excludes
 		log(
 			'Normal',
-			'recursive startup rsync: ',
+			'recursive full rsync: ',
 			config.source,
 			' -> ',
 			target
@@ -365,7 +481,7 @@ rsync.init = function
 
 		log(
 			'Normal',
-			'recursive startup rsync: ',
+			'recursive full rsync: ',
 			config.source,
 			' -> ',
 			target,
@@ -391,7 +507,7 @@ rsync.init = function
 
 		log(
 			'Normal',
-			'recursive startup rsync: ',
+			'recursive full rsync: ',
 			config.source,
 			' -> ',
 			target,
@@ -476,6 +592,7 @@ rsync.prepare = function
 
 	crsync._computed = { true }
 
+	--- @type any
 	local computed = crsync._computed
 
 	local computedN = 2
@@ -588,6 +705,12 @@ rsync.prepare = function
 		computedN = computedN  + 1
 	end
 
+	if crsync.copy_unsafe_links
+	then
+		computed[ computedN ] = '--copy-unsafe-links'
+		computedN = computedN  + 1
+	end
+
 	if crsync.groupmap
 	then
 		computed[ computedN ] = '--groupmap=' .. crsync.groupmap
@@ -682,7 +805,7 @@ rsync.exitcodes  = default.rsyncExitCodes
 rsync.rsync =
 {
 	-- The rsync binary to be called.
-	binary        = '/usr/bin/rsync',
+	binary        = 'rsync',
 	links         = true,
 	times         = true,
 	protect_args  = true
