@@ -13,6 +13,7 @@ then
 	error( 'default already loaded' )
 end
 
+--- @diagnostic disable-next-line: lowercase-global
 default = { }
 
 
@@ -38,9 +39,11 @@ default.checkgauge = {
 	action        =  true,
 	checkgauge    =  true,
 	collect       =  true,
+	crontab       =  true,
 	delay         =  true,
 	exitcodes     =  true,
 	init          =  true,
+	full          =  true,
 	maxDelays     =  true,
 	maxProcesses  =  true,
 	onAttrib      =  true,
@@ -49,9 +52,11 @@ default.checkgauge = {
 	onDelete      =  true,
 	onStartup     =  true,
 	onMove        =  true,
+	onFull        =  true,
 	prepare       =  true,
 	source        =  true,
 	target        =  true,
+	tunnel        =  true,
 }
 
 --
@@ -120,7 +125,14 @@ default.collect = function
 				agent.target,
 				' finished.'
 			)
-
+			if settings('onepass')
+			then
+				log(
+					'Normal', 
+					'onepass config set, exiting'
+				)
+				terminate( 0 )
+			end
 			return 'ok'
 		elseif rc == 'again'
 		then
