@@ -4,7 +4,7 @@
 -- This example uses local bash commands to keep two local
 -- directory trees in sync.
 --
-settings = {
+settings {
 	logfile         = "/tmp/lsyncd.log",
 	statusFile      = "/tmp/lsyncd.stat",
 	statusIntervall = 1,
@@ -15,12 +15,12 @@ settings = {
 -- for testing purposes. prefix can be used to slow commands down.
 -- prefix = "sleep 5 && "
 --
-prefix = ""
+local prefix = ""
 
 -----
 -- for testing purposes. uses bash command to hold local dirs in sync.
 --
-bash = {
+local bash = {
 	delay = 0,
 
 	maxProcesses = 1,
