@@ -13,6 +13,7 @@
 
 // some older machines need this to see pselect
 #define _DEFAULT_SOURCE 1
+#define _BSD_SOURCE 1
 #define _XOPEN_SOURCE 700
 #define _DARWIN_C_SOURCE 1
 
@@ -56,6 +57,7 @@ extern struct settings {
 	int log_facility; // The syslog facility
 	int log_level;    // -1 logs everything, 0 normal mode, LOG_ERROR errors only.
 	bool nodaemon;    // True if Lsyncd shall not daemonize.
+	bool onepass;     // True if Lsyncd should exit after first sync pass
 	char * pidfile;   // If not NULL Lsyncd writes its pid into this file.
 
 } settings;
