@@ -36,10 +36,10 @@ posix.sleep( 10 )
 cwriteln( 'killing the Lsyncd daemon' )
 
 posix.kill( pid )
-local _, exitmsg, lexitcode = posix.wait( lpid )
+local _, exitmsg, lexitcode = posix.wait( pid )
 cwriteln( 'Exitcode of Lsyncd = ', exitmsg, ' ', lexitcode )
 
-_, result, code = os.execute( 'diff -r ' .. srcdir .. ' ' .. trgdir )
+local result, code = execute( 'diff -r ' .. srcdir .. ' ' .. trgdir )
 
 if result == 'exit'
 then
