@@ -8,8 +8,6 @@ dofile( 'tests/testlib.lua' )
 cwriteln( '****************************************************************' )
 cwriteln( ' Testing default.rsyncssh with random data activity             ' )
 cwriteln( '****************************************************************' )
-cwriteln( '( this test needs passwordless ssh localhost access            )' )
-cwriteln( '( for current user                                             )' )
 
 local tdir, srcdir, trgdir = mktemps()
 
@@ -24,6 +22,8 @@ local pid = spawn(
 	'-nodaemon',
 	'-delay',
 	'5',
+	'-sshopts',
+	'-i tests/ssh/id_rsa -p 2468 -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null',
 	'-rsyncssh',
 	srcdir,
 	'localhost',
@@ -43,11 +43,11 @@ cwriteln( 'killing the Lsyncd daemon' )
 
 posix.kill(pid)
 
-local _, exitmsg, lexitcode = posix.wait( lpid )
+local _, exitmsg, lexitcode = posix.wait( pid )
 
 cwriteln( 'Exitcode of Lsyncd = ', exitmsg, ' ', lexitcode )
 
-_, result, code = os.execute( 'diff -r ' .. srcdir .. ' ' .. trgdir )
+local result, code = execute( 'diff -r ' .. srcdir .. ' ' .. trgdir )
 
 if result == 'exit'
 then
