@@ -115,7 +115,7 @@ cwriteln( 'waiting for Lsyncd to remove destination' )
 
 posix.sleep( 5 )
 
-_, result, code = os.execute( 'diff -urN ' .. srcdir .. ' ' .. trgdir )
+local result, code = execute( 'diff -urN ' .. srcdir .. ' ' .. trgdir )
 
 if result ~= 'exit' or code ~= 0
 then
@@ -137,7 +137,7 @@ testfiles( )
 cwriteln( 'killing started Lsyncd' )
 
 posix.kill( pid )
-local _, exitmsg, exitcode = posix.wait( lpid )
+local _, exitmsg, exitcode = posix.wait( pid )
 
 cwriteln( 'Exitcode of Lsyncd = ', exitmsg, ' ', exitcode );
 
