@@ -24,6 +24,14 @@ sync {
 	default.rsyncssh,
 	delay = 3,
 	host = 'localhost',
+	ssh = {
+       port= 2468,
+	   identityFile = "]] .. script_path() .. [[/ssh/id_rsa",
+	   options = {
+		StrictHostKeyChecking="no",
+		UserKnownHostsFile="/dev/null",
+	   },
+	},
 	source = ']]..srcdir..[[',
 	targetdir = ']]..trgdir..[[',
 	exclude = {
@@ -105,7 +113,7 @@ os.execute( 'rm -rf ' .. srcdir .. '/*' )
 cwriteln( 'waiting for Lsyncd to remove destination' )
 posix.sleep( 5 )
 
-_, result, code = os.execute( 'diff -urN '..srcdir..' '..trgdir )
+local result, code = execute( 'diff -urN '..srcdir..' '..trgdir )
 
 if result ~= 'exit' or code ~= 0
 then
@@ -121,7 +129,7 @@ testfiles( )
 
 cwriteln( 'killing started Lsyncd' )
 posix.kill( pid )
-local _, exitmsg, lexitcode = posix.wait( lpid )
+local _, exitmsg, lexitcode = posix.wait( pid )
 cwriteln( 'Exitcode of Lsyncd = ', exitmsg, ' ', lexitcode )
 posix.sleep( 1 )
 
