@@ -1,11 +1,22 @@
+--- @diagnostic disable: lowercase-global, need-check-nil
+
 -- common testing environment
 posix = require( 'posix' )
+string = require( 'string' )
+
+local sys_stat = require "posix.sys.stat"
 
 -- escape codes to colorize output on terminal
 local c1='\027[47;34m'
 
 local c0='\027[0m'
 
+-- compatibility with 5.1
+if table.unpack == nil then
+	--- @diagnostic disable-next-line: deprecated
+	table.unpack = unpack
+end
+
 --
 -- Writes colorized.
 --
@@ -15,14 +26,30 @@ function cwriteln
 	io.write( c0, '\n' )
 end
 
+local lver = string.gmatch(_VERSION, "Lua (%d).(%d)")
+_LUA_VERSION_MAJOR, _LUA_VERSION_MINOR = lver()
+_LUA_VERSION_MAJOR   = tonumber(_LUA_VERSION_MAJOR)
+_LUA_VERSION_MINOR  = tonumber(_LUA_VERSION_MINOR)
+
+-- Compatibility execute function
+function execute(...)
+	if _LUA_VERSION_MAJOR <= 5 and
+	   _LUA_VERSION_MINOR < 2 then
+		local rv = os.execute(...)
+		return "exit", rv
+	else
+		local ok, why, code = os.execute(...)
+		return why, code
+	end
+end
+
 --
 -- Initializes the pseudo random generator
 --
 -- If environment variable 'SEED' is set,
 -- that one is used seed.
 --
-local seed = os.getenv( 'SEED')  or os.time( )
-
+local seed = tonumber(os.getenv( 'SEED'))  or os.time( )
 math.randomseed( seed )
 
 cwriteln( 'random seed: ', seed )
@@ -36,6 +63,10 @@ function mktempd
 ( )
 	local f = io.popen( 'mktemp -td ltest.XXX', 'r' )
 
+	if f == nil then
+		return error("can't create testing directory")
+	end
+
 	local s = f:read( '*a' )
 
 	f:close( )
@@ -73,7 +104,8 @@ end
 function writefile
 (
 	filename,
-	text
+	text,
+	mode
 )
 	local f = io.open( filename, 'w' )
 
@@ -87,16 +119,132 @@ function writefile
 	f:write( '\n' )
 	f:close( )
 
+	if mode ~= nil then
+		posix.chmod(filename, mode)
+	end
+
 	return true
 end
 
+function splitpath(P)
+    local i = #P
+    local ch = P:sub(i,i)
+    while i > 0 and ch ~= "/" do
+        i = i - 1
+        ch = P:sub(i,i)
+    end
+    if i == 0 then
+        return '',P
+    else
+        return P:sub(1,i-1), P:sub(i+1)
+    end
+end
+
+function isabs(p)
+	return string.sub(p, 1, 2) == "/"
+end
+
+function abspath(P,pwd)
+	P = P:gsub('[\\/]$','')
+	if not isabs(P) then
+		local rv = posix.unistd.getcwd() .. "/" .. P
+		return rv
+	end
+	return P
+end
+
+
+function script_path()
+	-- local str = debug.getinfo(2, "S").source:sub(2)
+	-- return str:match("(.*/)")
+	local dir, file = splitpath(abspath(debug.getinfo(1).short_src))
+	return dir
+end
+
+function which(exec)
+	local path = os.getenv("PATH")
+	for match in (path..':'):gmatch("(.-)"..':') do
+		local fname = match..'/'..exec
+        local s = sys_stat.stat(fname)
+		if s ~= nil then
+			return fname
+		end
+    end
+end
+
+--
+-- Starts test ssh server
+--
+function startSshd()
+	-- local f = io.open(script_path() .. "ssh/sshd.pid", 'r')
+
+	-- if f
+	-- then
+	-- 	return false
+	-- end
+
+	cwriteln(arg[0])
+	cwriteln(script_path() ..  "ssh/sshd_config")
+
+	local sshdPath = script_path() .. "/ssh/"
+
+	if posix.stat( sshdPath ) == nil then
+		cwriteln("setup ssh server in " .. sshdPath)
+		posix.mkdir(sshdPath)
+		os.execute("ssh-keygen -t rsa -N '' -f" .. sshdPath .. "id_rsa")
+		os.execute("cp ".. sshdPath .. "id_rsa.pub ".. sshdPath .. "authorized_keys")
+		os.execute("ssh-keygen -t rsa -N '' -f ".. sshdPath .. "ssh_host_rsa_key")
+		cwriteln("done")
+	end
+
+	local f = io.open( sshdPath ..  "sshd_config", 'w')
+	local cfg = [[
+		Port 2468
+		HostKey ]] .. sshdPath .. [[ssh_host_rsa_key
+		AuthorizedKeysFile ]] .. sshdPath .. [[authorized_keys
+		ChallengeResponseAuthentication no
+		UsePAM no
+		#Subsystem   sftp    /usr/lib/ssh/sftp-server
+		PidFile ]] .. sshdPath .. [[sshd.pid
+	]]
+	cwriteln("Use ssh config: "..cfg)
+	f:write(cfg)
+	f:close()
+	--local which = io.popen("which sshd")
+	exePath = which('sshd')
+	cwriteln("Using sshd: "..exePath)
+
+	local pid = spawn(exePath, "-D", "-e", "-f", sshdPath .. "sshd_config")
+	cwriteln( 'spawned sshd server: ' .. pid)
+
+	return true
+end
+
+
+function strip(s)
+	return s:match "^%s*(.-)%s*$"
+end
+
+--
+-- Stop test ssh server
+--
+function stopSshd()
+	local f = io.open(script_path() .. "/ssh/sshd.pid", 'r')
+
+	if not f
+	then
+		return false
+	end
+	pid = strip(f:read("*a"))
+	posix.kill(tonumber(pid))
+end
+
 --
 -- Spawns a subprocess.
 --
 -- Returns the processes pid.
 --
-function spawn
-(...)
+function spawn(...)
 	args = { ... }
 
 	cwriteln( 'spawning: ', table.concat( args, ' ' ) )
@@ -499,4 +647,59 @@ function churn
 	end
 end
 
+-- check if tables are equal
+function isTableEqual(o1, o2, ignore_mt)
+    if o1 == o2 then return true end
+    local o1Type = type(o1)
+    local o2Type = type(o2)
+    if o1Type ~= o2Type then return false end
+    if o1Type ~= 'table' then return false end
 
+    if not ignore_mt then
+        local mt1 = getmetatable(o1)
+        if mt1 and mt1.__eq then
+            --compare using built in method
+            return o1 == o2
+        end
+    end
+
+    local keySet = {}
+
+    for key1, value1 in pairs(o1) do
+        local value2 = o2[key1]
+        if value2 == nil or isTableEqual(value1, value2, ignore_mt) == false then
+            return false
+        end
+        keySet[key1] = true
+    end
+
+    for key2, _ in pairs(o2) do
+        if not keySet[key2] then return false end
+    end
+    return true
+end
+
+--
+-- Tests if the filename exists
+-- fails if this is different to expect.
+--
+function testfile
+	(
+		filename,
+		expect
+	)
+	local stat, err = posix.stat( filename )
+
+	if stat and not expect
+	then
+		cwriteln( 'failure: ', filename, ' should be filtered')
+
+		os.exit( 1 )
+	end
+
+	if not stat and expect
+	then
+		cwriteln( 'failure: ', filename, ' should not be filtered' )
+		os.exit( 1 )
+	end
+end
\ No newline at end of file
